import tempfile
from datetime import date, datetime

import xlsxwriter

def get_filename(program, exportname, extension=None):
    today = str(date.today().strftime("%Y-%m-%d"))
    program = program.upper()
    filename = program + "_" + exportname + "_" + today
    if extension:
        filename += "." + extension
    return filename



def decode(value):
    if isinstance(value, str):
        return unicode(value, "utf-8")
    return value


def writerow(ws, row, rownr, format1):
    for i, value in enumerate(row):
        if isinstance(value, date) or isinstance(value, datetime):
            ws.write(rownr, i, decode(value), format1)
        else:
            ws.write(rownr, i, decode(value))


def export_xlsx_as_string(rows, metadata):
    with tempfile.TemporaryFile() as tmpfile:
        format_options, workbook, worksheet = configure_workbook(tmpfile)
        write(format_options, rows, worksheet)
        write_metadata(workbook, metadata, format_options)
        workbook.close()
        tmpfile.seek(0)
        xlsx_str = tmpfile.read()
    return xlsx_str


def write(format_options, rows, worksheet):
    """
    writes rows to worksheet
    :param format_options:
    :param rows:
    :param worksheet:
    :return:
    """
    for k, row in enumerate(rows):
        writerow(worksheet, row, k, format_options)


def write_metadata(workbook, metadata, format_options):
    sheet2 = workbook.add_worksheet("Metadaten")
    write(format_options, metadata, sheet2)


def configure_workbook(tmpfile):
    """
    configures a tmpfile as excel workbook
    :param tmpfile:
    :return:
    """
    workbook = xlsxwriter.Workbook(tmpfile,
                                   {'default_date_format': 'dd.mm.yyyy',
                                    'strings_to_urls': False
                                   })
    format_options = workbook.add_format({'num_format': 'dd.mm.yyyy',
                                          'font_size': 9})
    worksheet = workbook.add_worksheet("Rohdaten")
    return format_options, workbook, worksheet


def export_xlsx(request, result, name, program, metadata):
    excel = export_xlsx_as_string(result, metadata)

    response = request.response
    response.body = excel
    response.content_type = ("application/vnd.openxmlformats-officedocument."
                             "spreadsheetml.sheet")
    filename = get_filename(program, "{}".format(name), "xlsx")
    response.content_disposition = 'attachment; filename="{}"'.format(filename)
    return response

