# -*- coding: utf-8 -*-
from datetime import date
import os.path
import tempfile
import zipfile as ziplib
from efa.lib.export_formats.excelexport import export_xlsx_as_string, get_filename


def get_template_file_by_request(request, exportname):
    if exportname == "Teilnehmende":
        template_file  = request.registry.settings.get(
                'efa.participant_template')
    else:
        template_file = request.registry.settings.get(
                'efa.participation_template')
    if template_file is None:
        raise ValueError("efa.participation_template or efa.participant_template not set.")
    return (os.path.basename(template_file), template_file)


def export_zip(request, rows, exportname, program, metadata):
    excel = export_xlsx_as_string(rows, metadata)

    result_filename = get_filename(program,
                                   "{}_Auswertungsvorlage".format(exportname),
                                   "zip")
    arcname, template = get_template_file_by_request(request, exportname)

    with tempfile.TemporaryFile() as temp_file:
        with ziplib.ZipFile(temp_file, 'w') as zipfile:
            zipfile.writestr(program.upper() + "_" + exportname + ".xlsx", excel)
            zipfile.write(template, arcname)
        temp_file.seek(0)
        zip_stream = temp_file.read()

    response = request.response
    response.body = zip_stream
    response.content_type = "application/zip"
    response.content_disposition = 'attachment; filename="%s"' % result_filename

    return response
